@@ -7,8 +7,9 @@ class Component():
     A class for defining the methods that all gradio input and output components should have.
     """
 
-    def __init__(self, label):
+    def __init__(self, label, requires_permissions=False):
         self.label = label
+        self.requires_permissions = requires_permissions
 
     def __str__(self):
         return self.__repr__()
@@ -38,28 +39,36 @@ class Component():
         """
         return data
 
-    def restore_flagged(self, data):
+    def restore_flagged(self, dir, data, encryption_key):
         """
         Restores flagged data from logs
         """
         return data
 
     def save_flagged_file(self, dir, label, data, encryption_key):
+        if data is None:
+            return None
         file = processing_utils.decode_base64_to_file(data, encryption_key)
+        label = "".join([char for char in label if char.isalnum() or char in "._- "])
         old_file_name = file.name
         output_dir = os.path.join(dir, label)
         if os.path.exists(output_dir):
             file_index = len(os.listdir(output_dir))
         else:
-            os.mkdir(output_dir)
+            os.makedirs(output_dir)
             file_index = 0
         new_file_name = str(file_index)
         if "." in old_file_name:
             uploaded_format = old_file_name.split(".")[-1].lower()
             new_file_name +=  "." + uploaded_format
+        file.close()
         shutil.move(old_file_name, os.path.join(dir, label, new_file_name))
         return label + "/" + new_file_name
 
+    def restore_flagged_file(self, dir, file, encryption_key):
+        data = processing_utils.encode_file_to_base64(os.path.join(dir, file), encryption_key=encryption_key)
+        return {"name": file, "data": data}
+
     @classmethod
     def get_all_shortcut_implementations(cls):
         shortcuts = {}
