@@ -2,51 +2,54 @@
 Defines helper methods useful for setting up ports, launching servers, and handling `ngrok`
 """
 
-import os
-import socket
-import threading
 from flask import Flask, request, session, jsonify, abort, send_file, render_template, redirect
 from flask_cachebuster import CacheBuster
 from flask_login import LoginManager, login_user, current_user, login_required
 from flask_cors import CORS
-import threading
-import pkg_resources
-import datetime
-import time
-import json
-import urllib.request
-from shutil import copyfile
-import requests
-import sys
-import csv
-import logging
-from gradio.embeddings import calculate_similarity, fit_pca_to_embeddings, transform_with_pca
-from gradio.tunneling import create_tunnel
-from gradio import encryptor
-from gradio import queue
 from functools import wraps
+import inspect
 import io
+import json
+import logging
+import os
+import pkg_resources
+import requests
+import socket
+import sys
+import threading
+import time
+import traceback
+import urllib.parse
+import urllib.request
+from werkzeug.security import safe_join
+from werkzeug.serving import make_server
 
-INITIAL_PORT_VALUE = int(os.getenv(
-    'GRADIO_SERVER_PORT', "7860"))  # The http server will try to open on port 7860. If not available, 7861, 7862, etc.
-TRY_NUM_PORTS = int(os.getenv(
-    'GRADIO_NUM_PORTS', "100"))  # Number of ports to try before giving up and throwing an exception.
-LOCALHOST_NAME = os.getenv(
-    'GRADIO_SERVER_NAME', "127.0.0.1")
+from gradio import encryptor, queue
+from gradio.tunneling import create_tunnel
+from gradio.process_examples import load_from_cache, process_example
+
+# By default, the http server will try to open on port 7860. If not available, 7861, 7862, etc.
+INITIAL_PORT_VALUE = int(os.getenv('GRADIO_SERVER_PORT', "7860"))  
+# Number of ports to try before giving up and throwing an exception.
+TRY_NUM_PORTS = int(os.getenv('GRADIO_NUM_PORTS', "100"))  
+LOCALHOST_NAME = os.getenv('GRADIO_SERVER_NAME', "127.0.0.1")
 GRADIO_API_SERVER = "https://api.gradio.app/v1/tunnel-request"
 GRADIO_FEATURE_ANALYTICS_URL = "https://api.gradio.app/gradio-feature-analytics/"
 
-STATIC_TEMPLATE_LIB = pkg_resources.resource_filename("gradio", "frontend/")
-STATIC_PATH_LIB = pkg_resources.resource_filename("gradio", "frontend/static")
+STATIC_TEMPLATE_LIB = pkg_resources.resource_filename("gradio", "templates/")
+STATIC_PATH_LIB = pkg_resources.resource_filename("gradio", "templates/frontend/static")
 VERSION_FILE = pkg_resources.resource_filename("gradio", "version.txt")
+
 with open(VERSION_FILE) as version_file:
     GRADIO_STATIC_ROOT = "https://gradio.s3-us-west-2.amazonaws.com/" + \
-        version_file.read().strip() + "/static/"
+        version_file.read() + "/static/"
 
 app = Flask(__name__,
             template_folder=STATIC_TEMPLATE_LIB,
             static_folder="",
             static_url_path="/none/")
+app.url_map.strict_slashes = False
+
 CORS(app)
 cache_buster = CacheBuster(
     config={'extensions': ['.js', '.css'], 'hash_size': 5})
@@ -127,24 +130,24 @@ def get_first_available_port(initial, final):
 @app.route("/", methods=["GET"])
 @login_check
 def main():
-    session.clear()
     session["state"] = None
-    return render_template("index.html", config=app.interface.config)
+    return render_template("frontend/index.html", config=app.interface.config)
 
 
 @app.route("/static/<path:path>", methods=["GET"])
 def static_resource(path):
-    if app.interface.share or os.getenv("GRADIO_TEST_MODE"):
+    if app.interface.share:
         return redirect(GRADIO_STATIC_ROOT + path)
     else:
-        return send_file(os.path.join(STATIC_PATH_LIB, path))
+        return send_file(safe_join(STATIC_PATH_LIB, path))
 
 
+# TODO(@aliabid94): this throws a 500 error if app.auth is None (should probalbly just redirect to '/')
 @app.route('/login', methods=["GET", "POST"])
 def login():
     if request.method == "GET":
         config = get_config()
-        return render_template("index.html", config=config)
+        return render_template("frontend/index.html", config=config)
     elif request.method == "POST":
         username = request.form.get("username")
         password = request.form.get("password")
@@ -156,6 +159,39 @@ def login():
             return abort(401)
 
 
+@app.route("/api/", methods=["GET"])
+def api_docs():
+    inputs = [type(inp) for inp in app.interface.input_components]
+    outputs = [type(out) for out in app.interface.output_components]
+    input_types_doc, input_types = get_types(inputs, "input")
+    output_types_doc, output_types = get_types(outputs, "output")
+    input_names = [type(inp).__name__ for inp in app.interface.input_components]
+    output_names = [type(out).__name__ for out in app.interface.output_components]
+    if app.interface.examples is not None:
+        sample_inputs = app.interface.examples[0]
+    else:
+        sample_inputs = [inp.generate_sample() for inp in app.interface.input_components]
+    docs = {
+        "inputs": input_names,
+        "outputs": output_names,
+        "len_inputs": len(inputs),
+        "len_outputs": len(outputs),
+        "inputs_lower": [name.lower() for name in input_names],
+        "outputs_lower": [name.lower() for name in output_names],
+        "input_types": input_types,
+        "output_types": output_types,
+        "input_types_doc": input_types_doc,
+        "output_types_doc": output_types_doc,
+        "sample_inputs": sample_inputs,
+        "auth": app.interface.auth,
+        "local_login_url": urllib.parse.urljoin(
+            app.interface.local_url, "login"),
+        "local_api_url": urllib.parse.urljoin(
+            app.interface.local_url, "api/predict")
+    }
+    return render_template("api_docs.html", **docs)
+
+
 @app.route("/config/", methods=["GET"])
 def get_config():
     if app.interface.auth is None or current_user.is_authenticated:
@@ -173,203 +209,58 @@ def enable_sharing(path):
     return jsonify(success=True)
 
 
+@app.route("/shutdown", methods=['GET'])
+def shutdown():
+    shutdown_func = request.environ.get('werkzeug.server.shutdown')
+    if shutdown_func is None:
+        raise RuntimeError('Not running werkzeug')
+    shutdown_func()
+    return "Shutting down..."
+
+
 @app.route("/api/predict/", methods=["POST"])
 @login_check
 def predict():
-    raw_input = request.json["data"]
-    prediction, durations = app.interface.process(raw_input)
-    avg_durations = []
-    for i, duration in enumerate(durations):
-        app.interface.predict_durations[i][0] += duration
-        app.interface.predict_durations[i][1] += 1
-        avg_durations.append(app.interface.predict_durations[i][0] 
-            / app.interface.predict_durations[i][1])
-    app.interface.config["avg_durations"] = avg_durations
-    output = {"data": prediction, "durations": durations, "avg_durations": avg_durations}
-    if app.interface.allow_flagging == "auto":
-        try:
-            flag_index = flag_data(raw_input, prediction, 
+    request_data = request.get_json()
+    flag_index = None
+    if request_data.get("example_id") != None:
+        example_id = request_data["example_id"]
+        if app.interface.cache_examples:
+            prediction = load_from_cache(app.interface, example_id)
+            durations = None
+        else:
+            prediction, durations = process_example(app.interface, example_id)
+    else:
+        raw_input = request_data["data"]
+        if app.interface.show_error:
+            try:
+                prediction, durations = app.interface.process(raw_input)
+            except BaseException as error:
+                traceback.print_exc()
+                return jsonify({"error": str(error)}), 500
+        else:
+            prediction, durations = app.interface.process(raw_input)
+        if app.interface.allow_flagging == "auto":
+            flag_index = app.interface.flagging_callback.flag(app.interface, raw_input, prediction,
                 flag_option=(None if app.interface.flagging_options is None else ""), 
                 username=current_user.id if current_user.is_authenticated else None)
-            output["flag_index"] = flag_index
-        except Exception as e:
-            print(str(e))
-            pass
-    return jsonify(output)
+    output = {
+        "data": prediction, 
+        "durations": durations, 
+        "avg_durations": app.interface.config.get("avg_durations"),
+        "flag_index": flag_index
+    }
+    return output
 
 
-def log_feature_analytics(feature):
-    if app.interface.analytics_enabled:
-        try:
-            requests.post(GRADIO_FEATURE_ANALYTICS_URL,
-                          data={
-                              'ip_address': IP_ADDRESS,
-                              'feature': feature}, timeout=3)
-        except (requests.ConnectionError, requests.exceptions.ReadTimeout):
-            pass  # do not push analytics if no network
-
-
-@app.route("/api/score_similarity/", methods=["POST"])
-@login_check
-def score_similarity():
-    raw_input = request.json["data"]
-
-    preprocessed_input = [input_interface.preprocess(raw_input[i])
-                          for i, input_interface in enumerate(app.interface.input_components)]
-    input_embedding = app.interface.embed(preprocessed_input)
-    scores = list()
-
-    for example in app.interface.examples:
-        preprocessed_example = [iface.preprocess(iface.preprocess_example(example))
-                                for iface, example in zip(app.interface.input_components, example)]
-        example_embedding = app.interface.embed(preprocessed_example)
-        scores.append(calculate_similarity(input_embedding, example_embedding))
-    log_feature_analytics('score_similarity')
-    return jsonify({"data": scores})
-
-
-@app.route("/api/view_embeddings/", methods=["POST"])
-@login_check
-def view_embeddings():
-    sample_embedding = []
-    if "data" in request.json:
-        raw_input = request.json["data"]
-        preprocessed_input = [input_interface.preprocess(raw_input[i])
-                              for i, input_interface in enumerate(app.interface.input_components)]
-        sample_embedding.append(app.interface.embed(preprocessed_input))
-
-    example_embeddings = []
-    for example in app.interface.examples:
-        preprocessed_example = [iface.preprocess(iface.preprocess_example(example))
-                                for iface, example in zip(app.interface.input_components, example)]
-        example_embedding = app.interface.embed(preprocessed_example)
-        example_embeddings.append(example_embedding)
-
-    pca_model, embeddings_2d = fit_pca_to_embeddings(
-        sample_embedding + example_embeddings)
-    sample_embedding_2d = embeddings_2d[:len(sample_embedding)]
-    example_embeddings_2d = embeddings_2d[len(sample_embedding):]
-    app.pca_model = pca_model
-    log_feature_analytics('view_embeddings')
-    return jsonify({"sample_embedding_2d": sample_embedding_2d, "example_embeddings_2d": example_embeddings_2d})
-
-
-@app.route("/api/update_embeddings/", methods=["POST"])
-@login_check
-def update_embeddings():
-    sample_embedding, sample_embedding_2d = [], []
-    if "data" in request.json:
-        raw_input = request.json["data"]
-        preprocessed_input = [input_interface.preprocess(raw_input[i])
-                              for i, input_interface in enumerate(app.interface.input_components)]
-        sample_embedding.append(app.interface.embed(preprocessed_input))
-        sample_embedding_2d = transform_with_pca(
-            app.pca_model, sample_embedding)
-
-    return jsonify({"sample_embedding_2d": sample_embedding_2d})
-
-
-@app.route("/api/predict_examples/", methods=["POST"])
-@login_check
-def predict_examples():
-    example_ids = request.json["data"]
-    predictions_set = {}
-    for example_id in example_ids:
-        example_set = app.interface.examples[example_id]
-        processed_example_set = [iface.preprocess_example(example)
-                                 for iface, example in zip(app.interface.input_components, example_set)]
-        try:
-            predictions, _ = app.interface.process(processed_example_set)
-        except:
-            continue
-        predictions_set[example_id] = predictions
-    output = {"data": predictions_set}
-    return jsonify(output)
-
-
-def flag_data(input_data, output_data, flag_option=None, flag_index=None, username=None):
-    flag_path = os.path.join(app.cwd, app.interface.flagging_dir)
-    log_fp = "{}/log.csv".format(flag_path)
-    encryption_key = app.interface.encryption_key if app.interface.encrypt else None
-    is_new = not os.path.exists(log_fp)
-
-    if flag_index is None:
-        csv_data = []
-        for i, interface in enumerate(app.interface.input_components):
-            csv_data.append(interface.save_flagged(
-                flag_path, app.interface.config["input_components"][i]["label"], input_data[i], encryption_key))
-        for i, interface in enumerate(app.interface.output_components):
-            csv_data.append(interface.save_flagged(
-                flag_path, app.interface.config["output_components"][i]["label"], output_data[i], encryption_key) if output_data[i] is not None else "")
-        if flag_option is not None:
-            csv_data.append(flag_option)
-        if username is not None:
-            csv_data.append(username)
-        csv_data.append(str(datetime.datetime.now()))
-        if is_new:
-            headers = [interface["label"]
-                    for interface in app.interface.config["input_components"]]
-            headers += [interface["label"]
-                        for interface in app.interface.config["output_components"]]
-            if app.interface.flagging_options is not None:
-                headers.append("flag")
-            if username is not None:
-                headers.append("username")
-            headers.append("timestamp")
-
-    def replace_flag_at_index(file_content):
-        file_content = io.StringIO(file_content)
-        content = list(csv.reader(file_content))
-        header = content[0]
-        flag_col_index = header.index("flag")
-        content[flag_index][flag_col_index] = flag_option
-        output = io.StringIO()
-        writer = csv.writer(output)
-        writer.writerows(content)
-        return output.getvalue()
-
-    if app.interface.encrypt:
-        output = io.StringIO()
-        if not is_new:
-            with open(log_fp, "rb") as csvfile:
-                encrypted_csv = csvfile.read()
-                decrypted_csv = encryptor.decrypt(
-                    app.interface.encryption_key, encrypted_csv)
-                file_content = decrypted_csv.decode()
-                if flag_index is not None:
-                    file_content = replace_flag_at_index(file_content)
-                output.write(file_content)
-        writer = csv.writer(output)
-        if flag_index is None:
-            if is_new:
-                writer.writerow(headers)
-            writer.writerow(csv_data)
-        with open(log_fp, "wb") as csvfile:
-            csvfile.write(encryptor.encrypt(
-                app.interface.encryption_key, output.getvalue().encode()))
-    else:
-        if flag_index is None:
-            with open(log_fp, "a") as csvfile:
-                writer = csv.writer(csvfile)
-                if is_new:
-                    writer.writerow(headers)
-                writer.writerow(csv_data)
-        else:
-            with open(log_fp) as csvfile:
-                file_content = csvfile.read()
-                file_content = replace_flag_at_index(file_content)
-            with open(log_fp, "w") as csvfile:
-                csvfile.write(file_content)
-    with open(log_fp, "r") as csvfile:
-        line_count = len([None for row in csv.reader(csvfile)]) - 1
-    return line_count
-
 @app.route("/api/flag/", methods=["POST"])
 @login_check
 def flag():
     log_feature_analytics('flag')
     data = request.json['data']
-    flag_data(data['input_data'], data['output_data'], data.get("flag_option"), data.get("flag_index"), 
+    app.interface.flagging_callback.flag(
+        app.interface, data['input_data'], data['output_data'], 
+        data.get("flag_option"), data.get("flag_index"),
         current_user.id if current_user.is_authenticated else None)
     return jsonify(success=True)
 
@@ -391,23 +282,24 @@ def interpret():
 @login_check
 def file(path):
     if app.interface.encrypt and isinstance(app.interface.examples, str) and path.startswith(app.interface.examples):
-        with open(os.path.join(app.cwd, path), "rb") as encrypted_file:
+        with open(safe_join(app.cwd, path), "rb") as encrypted_file:
             encrypted_data = encrypted_file.read()
         file_data = encryptor.decrypt(
             app.interface.encryption_key, encrypted_data)
         return send_file(io.BytesIO(file_data), attachment_filename=os.path.basename(path))
     else:
-        return send_file(os.path.join(app.cwd, path))
+        return send_file(safe_join(app.cwd, path))
 
 
 @app.route("/api/queue/push/", methods=["POST"])
 @login_check
 def queue_push():
-    data = request.json["data"]
-    action = request.json["action"]
-    job_hash, queue_position = queue.push({"data": data}, action)
+    data = request.get_json()
+    action = data["action"]
+    job_hash, queue_position = queue.push(data, action)
     return {"hash": job_hash, "queue_position": queue_position}
 
+
 @app.route("/api/queue/status/", methods=["POST"])
 @login_check
 def queue_status():
@@ -415,7 +307,8 @@ def queue_status():
     status, data = queue.get_status(hash)
     return {"status": status, "data": data}
 
-def queue_thread(path_to_local_server):
+
+def queue_thread(path_to_local_server, test_mode=False):
     while True:
         try:
             next_job = queue.pop()
@@ -433,14 +326,28 @@ def queue_thread(path_to_local_server):
         except Exception as e:
             time.sleep(1)
             pass
+        if test_mode:
+            break
 
-def start_server(interface, server_name, server_port=None, auth=None, ssl=None):
-    if server_port is None:
-        server_port = INITIAL_PORT_VALUE
-    port = get_first_available_port(
-        server_port, server_port + TRY_NUM_PORTS
-    )
-    path_to_local_server = "http://{}:{}/".format(server_name, port)
+
+def start_server(interface, server_name=None, server_port=None, auth=None, ssl=None):
+    if server_name is None:
+        server_name = LOCALHOST_NAME
+    if server_port is None:  # if port is not specified, start at 7860 and search for first available port        
+        port = get_first_available_port(
+            INITIAL_PORT_VALUE, INITIAL_PORT_VALUE + TRY_NUM_PORTS
+        )
+    else:
+        try:
+            s = socket.socket()  # create a socket object
+            s.bind((LOCALHOST_NAME, server_port))  # Bind to the port to see if it's available (otherwise, raise OSError)
+            s.close()
+        except OSError:
+            raise OSError("Port {} is in use. If a gradio.Interface is running on the port, you can close() it or gradio.close_all().".format(server_port))
+        port = server_port
+
+    url_host_name = "localhost" if server_name == "0.0.0.0" else server_name
+    path_to_local_server = "http://{}:{}/".format(url_host_name, port)
     if auth is not None:
         if not callable(auth):
             app.auth = {account[0]: account[1] for account in auth}
@@ -460,15 +367,13 @@ def start_server(interface, server_name, server_port=None, auth=None, ssl=None):
         app.queue_thread.start()
     if interface.save_to is not None:
         interface.save_to["port"] = port
-    app_kwargs = {"port": port, "host": server_name}
+    app_kwargs = {"app": app, "port": port, "host": server_name}
     if ssl:
         app_kwargs["ssl_context"] = ssl
-    thread = threading.Thread(target=app.run,
-                              kwargs=app_kwargs,
-                              daemon=True)
+    server = make_server(**app_kwargs)
+    thread = threading.Thread(target=server.serve_forever, daemon=True)
     thread.start()
-
-    return port, path_to_local_server, app, thread
+    return port, path_to_local_server, app, thread, server
 
 def get_state():
     return session.get("state")
@@ -513,3 +418,35 @@ def url_ok(url):
                 return True
     except (ConnectionError, requests.exceptions.ConnectionError):
         return False
+    
+
+
+def get_types(cls_set, component):
+    docset = []
+    types = []
+    if component == "input":
+        for cls in cls_set:
+            doc = inspect.getdoc(cls.preprocess)
+            doc_lines = doc.split("\n")
+            docset.append(doc_lines[1].split(":")[-1])
+            types.append(doc_lines[1].split(")")[0].split("(")[-1])
+    else:
+        for cls in cls_set:
+            doc = inspect.getdoc(cls.postprocess)
+            doc_lines = doc.split("\n")
+            docset.append(doc_lines[-1].split(":")[-1])
+            types.append(doc_lines[-1].split(")")[0].split("(")[-1])
+    return docset, types
+
+
+def log_feature_analytics(feature):
+    if app.interface.analytics_enabled:
+        try:
+            requests.post(GRADIO_FEATURE_ANALYTICS_URL,
+                          data={
+                              'ip_address': IP_ADDRESS,
+                              'feature': feature}, timeout=3)
+        except (requests.ConnectionError, requests.exceptions.ReadTimeout):
+            pass  # do not push analytics if no network
+
+
