@@ -4,20 +4,22 @@ This module defines various classes that can serve as the `output` to an interfa
 automatically added to a registry, which allows them to be easily referenced in other parts of the code.
 """
 
+from posixpath import basename
 from gradio.component import Component
 import numpy as np
 import json
 from gradio import processing_utils
-import datetime
 import operator
 from numbers import Number
 import warnings
 import tempfile
-import scipy
 import os
 import pandas as pd
 import PIL
 from types import ModuleType
+from ffmpy import FFmpeg
+import requests
+
 
 class OutputComponent(Component):
     """
@@ -30,13 +32,18 @@ class OutputComponent(Component):
         """
         return y
 
+    def deserialize(self, x):
+        """
+        Convert from serialized output (e.g. base64 representation) from a call() to the interface to a human-readable version of the output (path of an image, etc.) 
+        """
+        return x
 
 
 class Textbox(OutputComponent):
     '''
     Component creates a textbox to render output text or number.
     Output type: Union[str, float, int]
-    Demos: hello_world.py, sentence_builder.py
+    Demos: hello_world, sentence_builder
     '''
 
     def __init__(self, type="auto", label=None):
@@ -62,19 +69,26 @@ class Textbox(OutputComponent):
         }
 
     def postprocess(self, y):
+        """
+        Parameters:
+        y (str): text output
+        Returns:
+        (Union[str, number]): output value
+        """
         if self.type == "str" or self.type == "auto":
             return str(y)
         elif self.type == "number":
             return y
         else:
-            raise ValueError("Unknown type: " + self.type + ". Please choose from: 'str', 'number'")
+            raise ValueError("Unknown type: " + self.type +
+                             ". Please choose from: 'str', 'number'")
 
 
 class Label(OutputComponent):
     '''
     Component outputs a classification label, along with confidence scores of top categories if provided. Confidence scores are represented as a dictionary mapping labels to scores between 0 and 1.
     Output type: Union[Dict[str, float], str, int, float]
-    Demos: image_classifier.py, main_note.py, titanic_survival.py
+    Demos: image_classifier, main_note, titanic_survival
     '''
 
     CONFIDENCES_KEY = "confidences"
@@ -91,6 +105,12 @@ class Label(OutputComponent):
         super().__init__(label)
 
     def postprocess(self, y):
+        """
+        Parameters:
+        y (Dict[str, float]): dictionary mapping label to confidence value 
+        Returns:
+        (Dict[label: str, confidences: List[Dict[label: str, confidence: number]]]): Object with key 'label' representing primary label, and key 'confidences' representing a list of label-confidence pairs
+        """
         if self.type == "label" or (self.type == "auto" and (isinstance(y, str) or isinstance(y, Number))):
             return {"label": str(y)}
         elif self.type == "confidences" or (self.type == "auto" and isinstance(y, dict)):
@@ -114,6 +134,20 @@ class Label(OutputComponent):
             raise ValueError("The `Label` output interface expects one of: a string label, or an int label, a "
                              "float label, or a dictionary whose keys are labels and values are confidences.")
 
+    def deserialize(self, y):
+        # 5 cases: (1): {'label': 'lion'}, {'label': 'lion', 'confidences':...}, {'lion': 0.46, ...}, 'lion', '0.46'
+        if self.type == "label" or (self.type == "auto" and (isinstance(y, str) or isinstance(y, int) or isinstance(y, float) or ('label' in y and not('confidences' in y.keys())))):
+            if isinstance(y, str) or isinstance(y, int) or isinstance(y, float):
+                return y
+            else:
+                return y['label']
+        elif self.type == "confidences" or self.type == "auto":
+            if ('confidences' in y.keys()) and isinstance(y['confidences'], list):
+                return {k['label']: k['confidence'] for k in y['confidences']}
+            else:
+                return y
+        raise ValueError("Unable to deserialize output: {}".format(y))
+
     @classmethod
     def get_shortcut_implementations(cls):
         return {
@@ -128,32 +162,32 @@ class Label(OutputComponent):
             return json.dumps({example["label"]: example["confidence"] for example in data["confidences"]})
         else:
             return data["label"]
-    
-    def restore_flagged(self, data):
+
+    def restore_flagged(self, dir, data, encryption_key):
         try:
             data = json.loads(data)
+            return self.postprocess(data)
+        except ValueError:
             return data
-        except:
-            return data
+
 
 class Image(OutputComponent):
     '''
     Component displays an output image. 
     Output type: Union[numpy.array, PIL.Image, str, matplotlib.pyplot, Tuple[Union[numpy.array, PIL.Image, str], List[Tuple[str, float, float, float, float]]]]
-    Demos: image_mod.py, webcam.py
+    Demos: image_mod, webcam
     '''
 
-    def __init__(self, type="auto", labeled_segments=False, plot=False, label=None):
+    def __init__(self, type="auto", plot=False, label=None):
         '''
         Parameters:
-        type (str): Type of value to be passed to component. "numpy" expects a numpy array with shape (width, height, 3), "pil" expects a PIL image object, "file" expects a file path to the saved image, "plot" expects a matplotlib.pyplot object, "auto" detects return type.
-        labeled_segments (bool): If True, expects a two-element tuple to be returned. The first element of the tuple is the image of format specified by type. The second element is a list of tuples, where each tuple represents a labeled segment within the image. The first element of the tuple is the string label of the segment, followed by 4 floats that represent the left-x, top-y, right-x, and bottom-y coordinates of the bounding box.
+        type (str): Type of value to be passed to component. "numpy" expects a numpy array with shape (width, height, 3), "pil" expects a PIL image object, "file" expects a file path to the saved image or a remote URL, "plot" expects a matplotlib.pyplot object, "auto" detects return type.
         plot (bool): DEPRECATED. Whether to expect a plot to be returned by the function.
         label (str): component name in interface.
         '''
-        self.labeled_segments = labeled_segments
         if plot:
-            warnings.warn("The 'plot' parameter has been deprecated. Set parameter 'type' to 'plot' instead.", DeprecationWarning)
+            warnings.warn(
+                "The 'plot' parameter has been deprecated. Set parameter 'type' to 'plot' instead.", DeprecationWarning)
             self.type = "plot"
         else:
             self.type = type
@@ -163,16 +197,17 @@ class Image(OutputComponent):
     def get_shortcut_implementations(cls):
         return {
             "image": {},
-            "segmented_image": {"labeled_segments": True},
             "plot": {"type": "plot"},
             "pil": {"type": "pil"}
         }
 
     def postprocess(self, y):
-        if self.labeled_segments:
-            y, coordinates = y
-        else:
-            coordinates = []
+        """
+        Parameters:
+        y (Union[numpy.array, PIL.Image, str, matplotlib.pyplot, Tuple[Union[numpy.array, PIL.Image, str], List[Tuple[str, float, float, float, float]]]]): image in specified format 
+        Returns:
+        (str): base64 url data
+        """
         if self.type == "auto":
             if isinstance(y, np.ndarray):
                 dtype = "numpy"
@@ -183,7 +218,8 @@ class Image(OutputComponent):
             elif isinstance(y, ModuleType):
                 dtype = "plot"
             else:
-                raise ValueError("Unknown type. Please choose from: 'numpy', 'pil', 'file', 'plot'.")
+                raise ValueError(
+                    "Unknown type. Please choose from: 'numpy', 'pil', 'file', 'plot'.")
         else:
             dtype = self.type
         if dtype in ["numpy", "pil"]:
@@ -191,55 +227,85 @@ class Image(OutputComponent):
                 y = np.array(y)
             out_y = processing_utils.encode_array_to_base64(y)
         elif dtype == "file":
-            out_y = processing_utils.encode_file_to_base64(y)
+            out_y = processing_utils.encode_url_or_file_to_base64(y)
         elif dtype == "plot":
             out_y = processing_utils.encode_plot_to_base64(y)
         else:
-            raise ValueError("Unknown type: " + dtype + ". Please choose from: 'numpy', 'pil', 'file', 'plot'.")
-        return out_y, coordinates
+            raise ValueError("Unknown type: " + dtype +
+                             ". Please choose from: 'numpy', 'pil', 'file', 'plot'.")
+        return out_y
+
+    def deserialize(self, x):
+        y = processing_utils.decode_base64_to_file(x).name
+        return y
 
     def save_flagged(self, dir, label, data, encryption_key):
-        """
-        Returns: (str) path to image file
-        """
-        return self.save_flagged_file(dir, label, data[0], encryption_key)
+        return self.save_flagged_file(dir, label, data, encryption_key)
+
+    def restore_flagged(self, dir, data, encryption_key):
+        return self.restore_flagged_file(dir, data, encryption_key)["data"]
 
 
 class Video(OutputComponent):
     '''
     Used for video output.     
     Output type: filepath
-    Demos: video_flip.py
+    Demos: video_flip
     '''
 
-    def __init__(self, label=None):
+    def __init__(self, type=None, label=None):
         '''
         Parameters:
+        type (str): Type of video format to be passed to component, such as 'avi' or 'mp4'. Use 'mp4' to ensure browser playability. If set to None, video will keep returned format.
         label (str): component name in interface.
         '''
+        self.type = type
         super().__init__(label)
 
     @classmethod
     def get_shortcut_implementations(cls):
         return {
             "video": {},
+            "playable_video": {"type": "mp4"}
         }
 
     def postprocess(self, y):
-        return processing_utils.encode_file_to_base64(y, type="video")
+        """
+        Parameters:
+        y (str): path to video 
+        Returns:
+        (str): base64 url data
+        """
+        returned_format = y.split(".")[-1].lower()
+        if self.type is not None and returned_format != self.type:
+            output_file_name = y[0: y.rindex(
+                ".") + 1] + self.type
+            ff = FFmpeg(
+                inputs={y: None},
+                outputs={output_file_name: None}
+            )
+            ff.run()
+            y = output_file_name
+        return {
+            "name": os.path.basename(y),
+            "data": processing_utils.encode_file_to_base64(y)
+        }
+
+    def deserialize(self, x):
+        return processing_utils.decode_base64_to_file(x).name
 
     def save_flagged(self, dir, label, data, encryption_key):
-        """
-        Returns: (str) path to image file
-        """
-        return self.save_flagged_file(dir, label, data, encryption_key)
+        return self.save_flagged_file(dir, label, data['data'], encryption_key)
+
+    def restore_flagged(self, dir, data, encryption_key):
+        return self.restore_flagged_file(dir, data, encryption_key)
 
 
 class KeyValues(OutputComponent):
     '''
     Component displays a table representing values for multiple fields. 
     Output type: Union[Dict, List[Tuple[str, Union[str, int, float]]]]
-    Demos: text_analysis.py
+    Demos: text_analysis
     '''
 
     def __init__(self, label=None):
@@ -250,6 +316,12 @@ class KeyValues(OutputComponent):
         super().__init__(label)
 
     def postprocess(self, y):
+        """
+        Parameters:
+        y (Union[Dict, List[Tuple[str, Union[str, int, float]]]]): dictionary or tuple list representing key value pairs 
+        Returns:
+        (List[Tuple[str, Union[str, number]]]): list of key value pairs
+        """
         if isinstance(y, dict):
             return list(y.items())
         elif isinstance(y, list):
@@ -263,11 +335,11 @@ class KeyValues(OutputComponent):
         return {
             "key_values": {},
         }
-    
+
     def save_flagged(self, dir, label, data, encryption_key):
         return json.dumps(data)
 
-    def restore_flagged(self, data):
+    def restore_flagged(self, dir, data, encryption_key):
         return json.loads(data)
 
 
@@ -276,21 +348,24 @@ class HighlightedText(OutputComponent):
     Component creates text that contains spans that are highlighted by category or numerical value.
     Output is represent as a list of Tuple pairs, where the first element represents the span of text represented by the tuple, and the second element represents the category or value of the text.
     Output type: List[Tuple[str, Union[float, str]]]
-    Demos: diff_texts.py, text_analysis.py
+    Demos: diff_texts, text_analysis
     '''
 
-    def __init__(self, color_map=None, label=None):
+    def __init__(self, color_map=None, label=None, show_legend=False):
         '''
         Parameters:
         color_map (Dict[str, str]): Map between category and respective colors
         label (str): component name in interface.
+        show_legend (bool): whether to show span categories in a separate legend or inline.
         '''
         self.color_map = color_map
+        self.show_legend = show_legend
         super().__init__(label)
 
     def get_template_context(self):
         return {
             "color_map": self.color_map,
+            "show_legend": self.show_legend,
             **super().get_template_context()
         }
 
@@ -301,12 +376,19 @@ class HighlightedText(OutputComponent):
         }
 
     def postprocess(self, y):
+        """
+        Parameters:
+        y (Union[Dict, List[Tuple[str, Union[str, int, float]]]]): dictionary or tuple list representing key value pairs 
+        Returns:
+        (List[Tuple[str, Union[str, number]]]): list of key value pairs
+
+        """
         return y
 
     def save_flagged(self, dir, label, data, encryption_key):
         return json.dumps(data)
 
-    def restore_flagged(self, data):
+    def restore_flagged(self, dir, data, encryption_key):
         return json.loads(data)
 
 
@@ -314,7 +396,7 @@ class Audio(OutputComponent):
     '''
     Creates an audio player that plays the output audio.
     Output type: Union[Tuple[int, numpy.array], str]
-    Demos: generate_tone.py, reverse_audio.py
+    Demos: generate_tone, reverse_audio
     '''
 
     def __init__(self, type="auto", label=None):
@@ -338,27 +420,39 @@ class Audio(OutputComponent):
         }
 
     def postprocess(self, y):
+        """
+        Parameters:
+        y (Union[Tuple[int, numpy.array], str]): audio data in requested format
+        Returns:
+        (str): base64 url data
+        """
         if self.type in ["numpy", "file", "auto"]:
             if self.type == "numpy" or (self.type == "auto" and isinstance(y, tuple)):
-                file = tempfile.NamedTemporaryFile(delete=False)
-                scipy.io.wavfile.write(file, y[0], y[1])                
+                sample_rate, data = y
+                file = tempfile.NamedTemporaryFile(
+                    prefix="sample", suffix=".wav", delete=False)
+                processing_utils.audio_to_file(sample_rate, data, file.name)
                 y = file.name
-            return processing_utils.encode_file_to_base64(y, type="audio", ext="wav")
+            return processing_utils.encode_url_or_file_to_base64(y)
         else:
-            raise ValueError("Unknown type: " + self.type + ". Please choose from: 'numpy', 'file'.")
+            raise ValueError("Unknown type: " + self.type +
+                             ". Please choose from: 'numpy', 'file'.")
+
+    def deserialize(self, x):
+        return processing_utils.decode_base64_to_file(x).name
 
     def save_flagged(self, dir, label, data, encryption_key):
-        """
-        Returns: (str) path to audio file
-        """
         return self.save_flagged_file(dir, label, data, encryption_key)
 
+    def restore_flagged(self, dir, data, encryption_key):
+        return self.restore_flagged_file(dir, data, encryption_key)["data"]
+
 
 class JSON(OutputComponent):
     '''
     Used for JSON output. Expects a JSON string or a Python object that is JSON serializable. 
     Output type: Union[str, Any]
-    Demos: zip_to_json.py
+    Demos: zip_to_json
     '''
 
     def __init__(self, label=None):
@@ -369,12 +463,17 @@ class JSON(OutputComponent):
         super().__init__(label)
 
     def postprocess(self, y):
+        """
+        Parameters:
+        y (Union[Dict, List, str]): JSON output
+        Returns:
+        (Union[Dict, List]): JSON output
+        """
         if isinstance(y, str):
             return json.dumps(y)
         else:
             return y
 
-
     @classmethod
     def get_shortcut_implementations(cls):
         return {
@@ -384,7 +483,7 @@ class JSON(OutputComponent):
     def save_flagged(self, dir, label, data, encryption_key):
         return json.dumps(data)
 
-    def restore_flagged(self, data):
+    def restore_flagged(self, dir, data, encryption_key):
         return json.loads(data)
 
 
@@ -392,7 +491,7 @@ class HTML(OutputComponent):
     '''
     Used for HTML output. Expects an HTML valid string. 
     Output type: str
-    Demos: text_analysis.py
+    Demos: text_analysis
     '''
 
     def __init__(self, label=None):
@@ -402,6 +501,14 @@ class HTML(OutputComponent):
         '''
         super().__init__(label)
 
+    def postprocess(self, x):
+        """
+        Parameters:
+        y (str): HTML output
+        Returns:
+        (str): HTML output
+        """
+        return x
 
     @classmethod
     def get_shortcut_implementations(cls):
@@ -414,7 +521,7 @@ class File(OutputComponent):
     '''
     Used for file output.     
     Output type: Union[file-like, str]
-    Demos: zip_two_files.py
+    Demos: zip_two_files
     '''
 
     def __init__(self, label=None):
@@ -424,7 +531,6 @@ class File(OutputComponent):
         '''
         super().__init__(label)
 
-
     @classmethod
     def get_shortcut_implementations(cls):
         return {
@@ -432,24 +538,30 @@ class File(OutputComponent):
         }
 
     def postprocess(self, y):
+        """
+        Parameters:
+        y (str): file path
+        Returns:
+        (Dict[name: str, size: number, data: str]): JSON object with key 'name' for filename, 'data' for base64 url, and 'size' for filesize in bytes 
+        """
         return {
             "name": os.path.basename(y),
-            "size": os.path.getsize(y), 
-            "data": processing_utils.encode_file_to_base64(y, header=False)
+            "size": os.path.getsize(y),
+            "data": processing_utils.encode_file_to_base64(y)
         }
 
     def save_flagged(self, dir, label, data, encryption_key):
-        """
-        Returns: (str) path to image file
-        """
         return self.save_flagged_file(dir, label, data["data"], encryption_key)
 
+    def restore_flagged(self, dir, data, encryption_key):
+        return self.restore_flagged_file(dir, data, encryption_key)
+
 
 class Dataframe(OutputComponent):
     """
     Component displays 2D output through a spreadsheet interface.
     Output type: Union[pandas.DataFrame, numpy.array, List[Union[str, float]], List[List[Union[str, float]]]]
-    Demos: filter_records.py, matrix_transpose.py, fraud_detector.py
+    Demos: filter_records, matrix_transpose, fraud_detector
     """
 
     def __init__(self, headers=None, max_rows=20, max_cols=None, overflow_row_behaviour="paginate", type="auto", label=None):
@@ -469,7 +581,6 @@ class Dataframe(OutputComponent):
         self.type = type
         super().__init__(label)
 
-
     def get_template_context(self):
         return {
             "headers": self.headers,
@@ -489,6 +600,12 @@ class Dataframe(OutputComponent):
         }
 
     def postprocess(self, y):
+        """
+        Parameters:
+        y (Union[pandas.DataFrame, numpy.array, List[Union[str, float]], List[List[Union[str, float]]]]): dataframe in given format
+        Returns:
+        (Dict[headers: List[str], data: List[List[Union[str, number]]]]): JSON object with key 'headers' for list of header names, 'data' for 2D array of string or numeric data
+        """
         if self.type == "auto":
             if isinstance(y, pd.core.frame.DataFrame):
                 dtype = "pandas"
@@ -505,25 +622,23 @@ class Dataframe(OutputComponent):
                 y = y.tolist()
             if len(y) == 0 or not isinstance(y[0], list):
                 y = [y]
-            return {"data": y} 
+            return {"data": y}
         else:
-            raise ValueError("Unknown type: " + self.type + ". Please choose from: 'pandas', 'numpy', 'array'.")
+            raise ValueError("Unknown type: " + self.type +
+                             ". Please choose from: 'pandas', 'numpy', 'array'.")
 
     def save_flagged(self, dir, label, data, encryption_key):
-        """
-        Returns: (List[List[Union[str, float]]]) 2D array
-        """
         return json.dumps(data["data"])
 
-    def restore_flagged(self, data):
-        return json.loads(data)
+    def restore_flagged(self, dir, data, encryption_key):
+        return {"data": json.loads(data)}
 
 
 class Carousel(OutputComponent):
     """
     Component displays a set of output components that can be scrolled through.
     Output type: List[List[Any]]
-    Demos: disease_report.py
+    Demos: disease_report
     """
 
     def __init__(self, components, label=None):
@@ -534,10 +649,10 @@ class Carousel(OutputComponent):
         '''
         if not isinstance(components, list):
             components = [components]
-        self.components = [get_output_instance(component) for component in components]
+        self.components = [get_output_instance(
+            component) for component in components]
         super().__init__(label)
 
-
     def get_template_context(self):
         return {
             "components": [component.get_template_context() for component in self.components],
@@ -545,6 +660,12 @@ class Carousel(OutputComponent):
         }
 
     def postprocess(self, y):
+        """
+        Parameters:
+        y (List[List[Any]]): carousel output
+        Returns:
+        (List[List[Any]]): 2D array, where each sublist represents one set of outputs or 'slide' in the carousel
+        """
         if isinstance(y, list):
             if len(y) != 0 and not isinstance(y[0], list):
                 y = [[z] for z in y]
@@ -556,34 +677,52 @@ class Carousel(OutputComponent):
                 output.append(output_row)
             return output
         else:
-            raise ValueError("Unknown type. Please provide a list for the Carousel.")
+            raise ValueError(
+                "Unknown type. Please provide a list for the Carousel.")
 
     def save_flagged(self, dir, label, data, encryption_key):
         return json.dumps([
             [
-                component.save_flagged(dir, f"{label}_{j}", data[i][j], encryption_key)
+                component.save_flagged(
+                    dir, f"{label}_{j}", data[i][j], encryption_key)
                 for j, component in enumerate(self.components)
-            ] for i, sample in enumerate(data)])
+            ] for i, _ in enumerate(data)])
 
+    def restore_flagged(self, dir, data, encryption_key):
+        return [
+            [
+                component.restore_flagged(dir, sample, encryption_key)
+                for component, sample in zip(self.components, sample_set)
+            ] for sample_set in json.loads(data)]
 
 
 def get_output_instance(iface):
     if isinstance(iface, str):
         shortcut = OutputComponent.get_all_shortcut_implementations()[iface]
         return shortcut[0](**shortcut[1])
+    # a dict with `name` as the output component type and other keys as parameters
+    elif isinstance(iface, dict):
+        name = iface.pop('name')
+        for component in OutputComponent.__subclasses__():
+            if component.__name__.lower() == name:
+                break
+        else:
+            raise ValueError("No such OutputComponent: {}".format(name))
+        return component(**iface)
     elif isinstance(iface, OutputComponent):
         return iface
     else:
         raise ValueError(
-            "Output interface must be of type `str` or "
-            "`OutputComponent`"
+            "Output interface must be of type `str` or `dict` or"
+            "`OutputComponent` but is {}".format(iface)
         )
 
+
 class Timeseries(OutputComponent):
     """
     Component accepts pandas.DataFrame.
     Output type: pandas.DataFrame
-    Demos: fraud_detector.py
+    Demos: fraud_detector
     """
 
     def __init__(self, x=None, y=None, label=None):
@@ -613,19 +752,23 @@ class Timeseries(OutputComponent):
         }
 
     def postprocess(self, y):
+        """
+        Parameters:
+        y (pandas.DataFrame): timeseries data
+        Returns:
+        (Dict[headers: List[str], data: List[List[Union[str, number]]]]): JSON object with key 'headers' for list of header names, 'data' for 2D array of string or numeric data
+        """
         return {
             "headers": y.columns.values.tolist(),
             "data": y.values.tolist()
 
         }
 
-
     def save_flagged(self, dir, label, data, encryption_key):
         """
         Returns: (List[List[Union[str, float]]]) 2D array
         """
         return json.dumps(data)
 
-    def restore_flagged(self, data):
+    def restore_flagged(self, dir, data, encryption_key):
         return json.loads(data)
-
