@@ -1,11 +1,17 @@
 from PIL import Image, ImageOps
 from io import BytesIO
 import base64
+import requests
 import tempfile
-import scipy.io.wavfile
-from scipy.fftpack import dct
+import shutil
+import os
 import numpy as np
 from gradio import encryptor
+import warnings
+import mimetypes
+with warnings.catch_warnings():
+    warnings.simplefilter("ignore") # Ignore pydub warning if ffmpeg is not installed
+    from pydub import AudioSegment
 
 #########################
 # IMAGE PRE-PROCESSING
@@ -16,15 +22,50 @@ def decode_base64_to_image(encoding):
     return Image.open(BytesIO(base64.b64decode(image_encoded)))
 
 
-def encode_file_to_base64(f, type="image", ext=None, header=True):
+def get_url_or_file_as_bytes(path):
+    try:
+        return requests.get(path).content
+    except (requests.exceptions.MissingSchema, requests.exceptions.InvalidSchema):
+        with open(path, "rb") as f:   
+            return f.read()
+
+
+def encode_url_or_file_to_base64(path):
+    try:
+        requests.get(path)
+        return encode_url_to_base64(path)
+    except (requests.exceptions.MissingSchema, requests.exceptions.InvalidSchema):
+        return encode_file_to_base64(path)
+
+def get_mimetype(filename):
+    mimetype = mimetypes.guess_type(filename)[0]
+    if mimetype is not None:
+        mimetype = mimetype.replace("x-wav", "wav")
+    return mimetype
+
+def get_extension(encoding):
+    encoding = encoding.replace("audio/wav", "audio/x-wav")
+    extension = mimetypes.guess_extension(mimetypes.guess_type(
+        encoding)[0])
+    if extension is not None and extension.startswith("."):
+        extension = extension[1:]
+    return extension
+
+def encode_file_to_base64(f, encryption_key=None):
     with open(f, "rb") as file:
         encoded_string = base64.b64encode(file.read())
+        if encryption_key:
+            encoded_string = encryptor.decrypt(encryption_key, encoded_string)
         base64_str = str(encoded_string, 'utf-8')
-        if not header:
-            return base64_str
-        if ext is None:
-            ext = f.split(".")[-1]
-        return "data:" + type + "/" + ext + ";base64," + base64_str
+        mimetype = get_mimetype(f)
+        return "data:" + (mimetype if mimetype is not None else "") + ";base64," + base64_str
+
+
+def encode_url_to_base64(url):
+    encoded_string = base64.b64encode(requests.get(url).content)
+    base64_str = str(encoded_string, 'utf-8')
+    mimetype = get_mimetype(url)
+    return "data:" + (mimetype if mimetype is not None else "") + ";base64," + base64_str
 
 
 def encode_plot_to_base64(plt):
@@ -62,35 +103,72 @@ def resize_and_crop(img, size, crop_type='center'):
         raise ValueError
     return ImageOps.fit(img, size, centering=center) 
 
+##################
+# Audio
+##################
+
+def audio_from_file(filename, crop_min=0, crop_max=100):
+    audio = AudioSegment.from_file(filename)
+    if crop_min != 0 or crop_max != 100:
+        audio_start = len(audio) * crop_min / 100
+        audio_end = len(audio) * crop_max / 100
+        audio = audio[audio_start : audio_end]
+    data = np.array(audio.get_array_of_samples())
+    if (audio.channels > 1):
+        data = data.reshape(-1, audio.channels)
+    return audio.frame_rate, data
+
+def audio_to_file(sample_rate, data, filename):
+    audio = AudioSegment(
+        data.tobytes(), 
+        frame_rate=sample_rate,
+        sample_width=data.dtype.itemsize, 
+        channels=(1 if len(data.shape) == 1 else data.shape[1])
+    )
+    audio.export(filename, format="wav")
+
 ##################
 # OUTPUT
 ##################
 
-def decode_base64_to_binary(encoding):
-    extension = None
-    if "," in encoding:
-        header, data = encoding.split(",")
-        header = header[5:]
-        if ";base64" in header:
-            header = header[0:header.index(";base64")]
-        if "/" in header:
-            extension = header[header.index("/") + 1:]
-    else:
-        data = encoding
-    return base64.b64decode(data), extension
 
-def decode_base64_to_file(encoding, encryption_key=None):
+def decode_base64_to_binary(encoding):
+    extension = get_extension(encoding)
+    data = encoding.split(",")[1]
+    return base64.b64decode(data), extension 
+
+def decode_base64_to_file(encoding, encryption_key=None, file_path=None):
     data, extension = decode_base64_to_binary(encoding)
+    prefix = None
+    if file_path is not None:
+        filename = os.path.basename(file_path)
+        prefix = filename
+        if "." in filename:
+            prefix = filename[0: filename.index(".")]
+            extension = filename[filename.index(".") + 1:]
     if extension is None:
-        file_obj = tempfile.NamedTemporaryFile(delete=False)
+        file_obj = tempfile.NamedTemporaryFile(delete=False, prefix=prefix)
     else:
-        file_obj = tempfile.NamedTemporaryFile(delete=False, suffix="."+extension)
+        file_obj = tempfile.NamedTemporaryFile(delete=False, prefix=prefix, suffix="."+extension)
     if encryption_key is not None:
         data = encryptor.encrypt(encryption_key, data)
     file_obj.write(data)
     file_obj.flush()
     return file_obj
 
+def create_tmp_copy_of_file(file_path):
+    file_name = os.path.basename(file_path)
+    prefix, extension = file_name, None
+    if "." in file_name:
+        prefix = file_name[0: file_name.index(".")]
+        extension = file_name[file_name.index(".") + 1:]
+    if extension is None:
+        file_obj = tempfile.NamedTemporaryFile(delete=False, prefix=prefix)
+    else:
+        file_obj = tempfile.NamedTemporaryFile(delete=False, prefix=prefix, suffix="."+extension)
+    shutil.copy2(file_path, file_obj.name)
+    return file_obj
+
 def _convert(image, dtype, force_copy=False, uniform=False):
     """
     Adapted from: https://github.com/scikit-image/scikit-image/blob/main/skimage/util/dtype.py#L510-L531
@@ -373,96 +451,3 @@ def _convert(image, dtype, force_copy=False, uniform=False):
     image = _scale(image, 8 * itemsize_in, 8 * itemsize_out, copy=False)
     image += imin_out
     return image.astype(dtype_out)
-
-
-##################
-# AUDIO FILES
-##################
-
-def generate_mfcc_features_from_audio_file(wav_filename=None,
-                                           pre_emphasis=0.95,
-                                           frame_size= 0.025,
-                                           frame_stride=0.01,
-                                           NFFT=512,
-                                           nfilt=40,
-                                           num_ceps=12,
-                                           cep_lifter=22,
-                                           sample_rate=None,
-                                           signal=None,
-                                           downsample_to=None):
-    """
-    Loads and preprocesses a .wav audio file (or alternatively, a sample rate & signal) into mfcc coefficients, the typical inputs to models.
-    Adapted from: https://haythamfayek.com/2016/04/21/speech-processing-for-machine-learning.html
-    :param wav_filename: string name of audio file to process.
-    :param pre_emphasis: a float factor, typically 0.95 or 0.97, which amplifies high frequencies.
-    :param frame_size: a float that is the length, in seconds, of time frame over which to take the fft.
-    :param frame_stride: a float that is the offset, in seconds, between consecutive time frames.
-    :param NFFT: The number of points in the short-time fft for each time frame.
-    :param nfilt: The number of filters on the Mel-scale to extract frequency bands.
-    :param num_ceps: the number of cepstral coefficients to retrain.
-    :param cep_lifter: the int factor, by which to de-emphasize higher-frequency.
-    :param sample_rate: optional param represnting sample rate that is used if `wav_filename` is not provided
-    :param signal: optional param representing sample data that is used if `wav_filename` is not provided
-    :param downsample_to: optional param. If provided, audio file is downsampled to this many frames.  
-    :return: a 3D numpy array of mfcc coefficients, of the shape 1 x num_frames x num_coeffs.
-    """
-    if (wav_filename is None) and (sample_rate is None or signal is None):
-        raise ValueError("Either a wav_filename must be provdied or a sample_rate and signal") 
-    elif wav_filename is None:
-        pass
-    else:
-        sample_rate, signal = scipy.io.wavfile.read(wav_filename)
-
-    if not(downsample_to is None):
-        signal = scipy.signal.resample(signal, downsample_to)    
-
-    emphasized_signal = np.append(signal[0], signal[1:] - pre_emphasis * signal[:-1])
-
-    frame_length, frame_step = frame_size * sample_rate, frame_stride * sample_rate  # Convert from seconds to samples
-    signal_length = len(emphasized_signal)
-    frame_length = int(round(frame_length))
-    frame_step = int(round(frame_step))
-    num_frames = int(np.ceil(float(np.abs(signal_length - frame_length)) / frame_step))  # Make sure that we have at least 1 frame
-
-    pad_signal_length = num_frames * frame_step + frame_length
-    z = np.zeros((pad_signal_length - signal_length))
-    pad_signal = np.append(emphasized_signal, z)  # Pad Signal to make sure that all frames have equal number of samples without truncating any samples from the original signal
-
-    indices = np.tile(np.arange(0, frame_length), (num_frames, 1)) + np.tile(np.arange(0, num_frames * frame_step, frame_step), (frame_length, 1)).T
-    frames = pad_signal[indices.astype(np.int32, copy=False)]
-
-    frames *= np.hamming(frame_length)
-    mag_frames = np.absolute(np.fft.rfft(frames, NFFT))  # Magnitude of the FFT
-    pow_frames = ((1.0 / NFFT) * ((mag_frames) ** 2))  # Power Spectrum
-
-    low_freq_mel = 0
-    high_freq_mel = (2595 * np.log10(1 + (sample_rate / 2) / 700))  # Convert Hz to Mel
-    mel_points = np.linspace(low_freq_mel, high_freq_mel, nfilt + 2)  # Equally spaced in Mel scale
-    hz_points = (700 * (10**(mel_points / 2595) - 1))  # Convert Mel to Hz
-    bin = np.floor((NFFT + 1) * hz_points / sample_rate)
-
-    fbank = np.zeros((nfilt, int(np.floor(NFFT / 2 + 1))))
-    for m in range(1, nfilt + 1):
-        f_m_minus = int(bin[m - 1])   # left
-        f_m = int(bin[m])             # center
-        f_m_plus = int(bin[m + 1])    # right
-
-        for k in range(f_m_minus, f_m):
-            fbank[m - 1, k] = (k - bin[m - 1]) / (bin[m] - bin[m - 1])
-        for k in range(f_m, f_m_plus):
-            fbank[m - 1, k] = (bin[m + 1] - k) / (bin[m + 1] - bin[m])
-    filter_banks = np.dot(pow_frames, fbank.T)
-    filter_banks = np.where(filter_banks == 0, np.finfo(float).eps, filter_banks)  # Numerical Stability
-    filter_banks = 20 * np.log10(filter_banks)  # dB
-
-    mfcc = dct(filter_banks, type=2, axis=1, norm='ortho')[:, 0: (num_ceps + 1)]  # Keep filters 1-13 by default.
-    (nframes, ncoeff) = mfcc.shape
-    n = np.arange(ncoeff)
-    lift = 1 + (cep_lifter / 2) * np.sin(np.pi * n / cep_lifter)
-    mfcc *= lift
-
-    filter_banks -= (np.mean(filter_banks, axis=0) + 1e-8)
-    mfcc -= (np.mean(mfcc, axis=0) + 1e-8)
-    return mfcc[np.newaxis, :, :]  # Create a batch dimension.
-
-
