@@ -11,14 +11,16 @@ import random
 import os
 
 current_dir = os.getcwd()
-os.environ["GRADIO_TEST_MODE"] = "1"
 
 LOCAL_HOST = "http://localhost:{}"
 GOLDEN_PATH = "test/golden/{}/{}.png"
 TOLERANCE = 0.1
 TIMEOUT = 10
 
-GAP_TO_SCREENSHOT = 1
+GAP_TO_SCREENSHOT = 2
+
+os.environ["GRADIO_ANALYTICS_ENABLED"] = "False"
+
 
 def wait_for_url(url):
     for i in range(TIMEOUT):
@@ -33,26 +35,26 @@ def wait_for_url(url):
 
 
 def diff_texts_thread(return_dict):
-    from demo.diff_texts import iface
+    from demo.diff_texts.run import iface
     iface.save_to = return_dict
     iface.launch()
 
 
 def image_mod_thread(return_dict):
-    from demo.image_mod import iface
+    from demo.image_mod.run import iface
     iface.examples = None
     iface.save_to = return_dict
     iface.launch()
 
 
 def longest_word_thread(return_dict):
-    from demo.longest_word import iface
+    from demo.longest_word.run import iface
     iface.save_to = return_dict
     iface.launch()
 
 
 def sentence_builder_thread(return_dict):
-    from demo.sentence_builder import iface
+    from demo.sentence_builder.run import iface
     iface.save_to = return_dict
     iface.launch()
 
@@ -97,7 +99,7 @@ class TestDemo(unittest.TestCase):
         elem.click()
         elem = WebDriverWait(driver, TIMEOUT).until(
             EC.presence_of_element_located((By.CSS_SELECTOR,
-                                            ".panel:nth-child(2) .component:nth-child(2) .output_text"))
+                                            ".panel:nth-child(2) .component:nth-child(2) .textfield"))
         )
 
         total_sleep = 0
@@ -105,7 +107,7 @@ class TestDemo(unittest.TestCase):
             time.sleep(0.2)
             total_sleep += 0.2
 
-        self.assertEqual(elem.text, "LeWant's tgo see a magic trick?!")
+        self.assertEqual(elem.text, "L+e+W-a-n-t'+s+ t-g+o see a magic trick?-!+")
         golden_img = os.path.join(current_dir, GOLDEN_PATH.format(
             "diff_texts", "magic_trick"))
         tmp = os.path.join(current_dir, "test/tmp/{}.png".format(
@@ -123,7 +125,7 @@ class TestDemo(unittest.TestCase):
                 (By.CSS_SELECTOR, ".panel:nth-child(1) .component:nth-child(1) .hidden_upload"))
         )
         cwd = os.getcwd()
-        rel = "demo/images/cheetah1.jpg"
+        rel = "test/test_files/cheetah1.jpg"
         elem.send_keys(os.path.join(cwd, rel))
         golden_img = os.path.join(current_dir, GOLDEN_PATH.format(
             "image_mod", "cheetah1"))
@@ -159,7 +161,7 @@ class TestDemo(unittest.TestCase):
         )
         elem.click()
         elem = WebDriverWait(driver, TIMEOUT).until(
-            EC.presence_of_element_located((By.CSS_SELECTOR, ".panel:nth-child(2) .component:nth-child(2) .output_class"))
+            EC.presence_of_element_located((By.CSS_SELECTOR, ".panel:nth-child(2) .component:nth-child(2) .output_class_without_confidences"))
         )
 
         total_sleep = 0
